<?php

/* $Id$ */

/**
 * This file implements the group summary report generation logic.
 * 
 * @package EnterTrack
 * @subpackage Reports_Module
 */

// We only execute main().
main();


/**
 * Processing entry point.  Top-level procedure.
 * 
 */
function main() {
//    ini_set( 'display_errors', 'on' );
//    ini_set( 'error_reporting', E_ALL );

    if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
        print "Direct module access forbidden.";
        exit;
    }
    
    $data = get_report();
    $totals = calc_totals( $data );
    display_report( aget($_POST, 'format'), $data, $totals );
}


/**
 * Verifies that all values in an array are numeric.
 * 
 */
function verify_numeric_array($array, $empty_allowed, $error_msg ) {
    if( !$empty_allowed && (!is_array( $array ) || !count( $array )) ) {
        print $error_msg;
        exit;
    }
    foreach( $array as $val ) {
        if( !is_numeric( $val ) ) {
            print $error_msg;
            exit;
        }
    }
}


/**
 * Generates a report.
 * 
 */
function get_report( ) {
    $gid_list = aget( $_POST, 'gid');
    verify_numeric_array( $gid_list, false, _('Invalid group selected.') );

    $groups_data = get_report_data( $gid_list, 
                                    aget( $_POST, 'sdate'), 
                                    aget( $_POST, 'edate') );
    return $groups_data;
}

/**
 * Report generation worker function.  Contains SQL logic.
 * 
 */
function get_report_data( $group_id_list, $start_date, $end_date ) {
    $params = array();

    $startDate = null;
    if ( $start_date ) {
      $startDate = $_ENV['i18n']->parse_datetime($start_date);
      if (!$startDate) {
        $_ENV['error']->push_error(_("Invalid start date."));
      }
    }
    
    $endDate = null;
    if ( $end_date ) {
      $endDate = $_ENV['i18n']->parse_datetime($end_date);
      if (!$endDate) {
        $_ENV['error']->push_error(_("Invalid end date."));
      }
    }

    $gid_list = implode( ',', $group_id_list );

    $time_now = time();

    $start_clause = $startDate ? " and i.opened >= '$startDate'" : '';
    $end_clause = $endDate ? " and i.opened <= '$endDate'" : '';

    $start_clause_closed = $startDate ? " and i.closed >= '$startDate'" : '';
    $end_clause_closed = $endDate ? " and i.closed <= '$endDate'" : '';


    $sql = <<< END
select
 g.gid,
 g.name,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause ) as num_issues_opened,
 (select count(*) from issue_groups ig, issues i, events e where e.issueid = ig.issueid and ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause ) as num_events,
 (select coalesce(sum(e.duration),0) from issue_groups ig, issues i, events e where e.issueid = ig.issueid and ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause ) as sum_events,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause and status in (2,3) ) as num_waiting,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause and status in (6) ) as num_longterm,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause and closed is not null ) as num_closed,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause and closed is null ) as num_active,
 (select count(*) from issue_groups ig, issues i where ig.gid = g.gid and ig.issueid = i.issueid $start_clause $end_clause and i.due_date < $time_now) as num_past_due_date
from 
 groups g
where
 g.gid in ($gid_list)
END;

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
    $result->free();
    $stmt->free();

    return $data;
}

/**
 * Calculates totals for each column in an input array.
 * 
 */
function calc_totals( $data ) {
    $totals = array();
    foreach( $data as $group ) {
        foreach( $group as $key => $amt ) {
            $totals[$key] = (int)(isset( $totals[$key] ) ? $totals[$key] + $amt : $amt);
        }
    }
    return $totals;
}

/**
 * Displays report in various formats
 * 
 */
function display_report( $format, $data, $totals ) {

    switch( trim($format) ) {
    case 'html': display_report_html( $data, $totals ); break;
    case 'csv': display_report_csv( $data, $totals ); break;
    case 'word': display_report_word( $data, $totals ); break;
    case 'excel': display_report_excel( $data, $totals ); break;
    case 'pdf': display_report_pdf( $data, $totals ); break;
    default: 
        $_ENV['error']->push_error(_("Unknown report format."));
        break;
    }
}


/**
 * Displays report in HTML via smarty.
 * 
 */
function display_report_html( $data, $totals ) {
    $_ENV['tpl']->assign('report', 'Groups Summary');
    $_ENV['tpl']->assign('data', $data);
    $_ENV['tpl']->assign('totals', $totals);
    $_ENV['tpl']->assign('time', time());
    $_ENV['tpl']->assign('user', $_SESSION['userid']);
    $_ENV['tpl']->assign('startDate', $_ENV['i18n']->parse_datetime(aget($_POST, 'sdate')));
    $_ENV['tpl']->assign('endDate', $_ENV['i18n']->parse_datetime(aget($_POST, 'edate')));
    $_ENV['tpl']->display("reports/group_summary_result.tpl");

}

/**
 * Displays report in CSV
 * 
 */
function display_report_csv( $data, $totals ) {

    header( "Content-Type: text/plain" );

    $fh = tmpfile();
    if( !$fh ) {
        $_ENV['error']->push_error(_("Unable to generate CSV output."));
        return false;
    }

    // print a header using array keys.
    if ( count( $data ) ) {
        fputcsv( $fh, array_keys( $data[0] ) );
    }

    foreach( $data as $row_data ) {
        fputcsv( $fh, $row_data );
    }
    fseek($fh, 0);

    $buf = stream_get_contents( $fh );

    fclose( $fh );

    echo $buf;
}


/**
 * Displays report in PDF
 * 
 */
function display_report_pdf( $data, $totals ) {
    echo "PDF Report Format Unimplemented";
}

/**
 * Displays report in MS Word Doc format
 * 
 */
function display_report_word( $data, $totals ) {
    echo "Word Report Format Unimplemented";
}

/**
 * Displays report in MS Excel format.
 * 
 */
function display_report_excel( $data, $totals ) {
    echo "Excel Report Format Unimplemented";
}

?>
